<?php
/**
 * Record driver view helper
 *
 * PHP version 5
 *
 * Copyright (C) Villanova University 2010.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2,
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * @category VuFind
 * @package  View_Helpers
 * @author   Demian Katz <demian.katz@villanova.edu>
 * @license  http://opensource.org/licenses/gpl-2.0.php GNU General Public License
 * @link     https://vufind.org/wiki/development Wiki
 */
namespace finc\View\Helper\Root;

use finc\Rewrite\EblRewrite;
use Symfony\Component\Config\Definition\Exception\Exception;

/**
 * Record driver view helper
 *
 * @category VuFind
 * @package  View_Helpers
 * @author   Demian Katz <demian.katz@villanova.edu>
 * @license  http://opensource.org/licenses/gpl-2.0.php GNU General Public License
 * @link     https://vufind.org/wiki/development Wiki
 */
class Record extends \VuFind\View\Helper\Root\Record
{
    /**
     * VuFind configuration
     *
     * @var \Zend\Config\Config
     */
    protected $config;

    /**
     * Authentication manager
     *
     * @var \VuFind\Auth\Manager
     */
    protected $manager;

    /**
     * URL helper
     *
     * @var \Zend\View\Helper\Url
     */
    protected $url;

    /**
     * Rewriter
     *
     * @var EblRewrite $rewrite
     */
    protected $rewrite;

    /**
     * Resolver configuration
     *
     * @var \Zend\Config\Config
     */
    protected $resolverConfig;

    /**
     * Constructor
     *
     * @param \Zend\Config\Config $config VuFind configuration
     * @param \Zend\View\Helper\Url $helper URL helper
     * @param \VuFind\Auth\Manager $manager AuthManager
     * @param mixed $rewrite
     * @param mixed $resolverConfig
     *
     */
    public function __construct(
        $config,
        \Zend\View\Helper\Url $helper,
        \VuFind\Auth\Manager $manager,
        $rewrite,
        $resolverConfig
    ) {
        parent::__construct($config);
        $this->url = $helper;
        $this->manager = $manager;
        $this->rewrite = $rewrite;
        $this->resolverConfig = $resolverConfig;
    }

    /**
     * Render a (list of) record icons.
     *
     * @param string $tpl Define alternative template for record icon. Default:
     *                      record-icon.phtml
     *
     * @return string
     */
    public function getRecordIcon($tpl = 'record-icon')
    {
        return $this->renderTemplate($tpl . '.phtml');
    }

    /**
     * Get the CSS class used to properly render an icon for given value
     *
     * @param string $value (identifier) Value to convert into CSS class
     * @param string $classfile Define alternative file for icon class without
     *                              suffix. Default: record-icon-class.phtml
     *
     * @return string
     */
    public function getRecordIconClass($value, $classfile = 'record-icon-class')
    {
        return $this->renderTemplate(
            $classfile . '.phtml',
            ['value' => $value]
        );
    }

    /**
     * Get the CSS class used to properly render an icon for given value
     *
     * @param string $classfile Define alternative file for icon text without
     *
     * @return string
     */
    public function getRecordIconText($classfile = 'record-icon-text')
    {
        return $this->renderTemplate(
            $classfile . '.phtml'
        );
    }

    /**
     * Returns if style based icons should be shown (if covers are disabled!)
     *
     * @return bool
     */
    public function showStyleBasedIcons()
    {
        return isset($this->config->Content->showStyleBasedIcons) ?
            $this->config->Content->showStyleBasedIcons : false;
    }

    /**
     * Get external access links to other ILS defined by config setting.
     *
     * @return     array     Associative array.
     * @throws     Exception Value of source ids has to be numeric.
     * @deprecated Deprecated due to View Helper ExternalCatalogueLink->getLinks()
     * @see        https://projekte.ub.uni-leipzig.de/issues/10543
     */
    public function getExternalAccessLinks()
    {
        $i = -1; // iterator of extUrls
        $extUrls = [];

        // if configuration empty return unprocessed
        if (!isset($this->config->ExternalAccess)
            || count($this->config->ExternalAccess) == 0
        ) {
            return [];
        }
        // if institutions empty return unprocessed
        $institutions = $this->driver->tryMethod('getInstitutions');
        if (!isset($institutions) || count($institutions) == 0) {
            return [];
        }
        foreach ($this->config->ExternalAccess as $recordType => $accessUrl) {
            $replaceId = null;
            // get identifier of record id type
            switch ($recordType) {
                case "id":
                    $replaceId = $this->driver->getUniqueID();
                    break;
                case "ppn":
                    $replaceId = $this->driver->tryMethod('getRID');
                    break;
                default:
                    $replaceId = null;
            }
            foreach ($accessUrl as $institution => $urlPattern) {
                // source_id filter
                if (is_array($urlPattern)) {
                    foreach ($urlPattern as $source_id => $pattern) {
                        $urlPattern = $pattern;
                        $sids = explode(',', $source_id);
                        if (!array_product(array_map('is_numeric', $sids))) {
                            throw new Exception(
                                'Value of source ids has to be numeric.'
                            );
                        }
                        $sourceID = $this->driver->tryMethod('getSourceID');
                        $replaceId = (isset($sourceID)
                            && true === in_array($sourceID, $sids))
                            ? $replaceId : null;
                        break;
                    }
                }

                // institution filter
                if (true === in_array($institution, $institutions)
                    && !empty($replaceId)
                ) {
                    $extUrls[++$i]['desc'] = $institution;
                    $extUrls[$i]['url'] = sprintf($urlPattern, $replaceId);
                }
            }
        }
        return $extUrls;
    }

    /**
     * Render the link of the specified type.
     *
     * @param string $type    Link type
     * @param string $lookfor String to search for at link
     *
     * @return string
     */
    public function getLink($type, $lookfor)
    {
        $lookfor = (
            $type == 'author'
                ? $this->removeAuthorDates($lookfor) : $lookfor
        );
        return parent::getLink($type, $lookfor);
    }

    /**
     * Get all the links associated with this record.  Returns an array of
     * associative arrays each containing 'desc' and 'url' keys.
     *
     * @param bool $openUrlActive Is there an active OpenURL on the page?
     *
     * @return array
     */
    public function getLinkDetails($openUrlActive = false)
    {
        $links = parent::getLinkDetails($openUrlActive);
        return $this->rewriteLinks($links);
    }

    /**
     * Map list of multiple dates to a view of date range as string e.g.
     * startdate - enddate Sorting of array should be managed by backend.
     *
     * @param array $dates Dates
     *
     * @return mixed
     */
    public function mapDateListToRangeView($dates)
    {
        if (is_array($dates)) {
            if (count($dates) == 1) {
                return $dates[0];
            } else {
                return array_shift($dates) . '-' . array_pop($dates);
            }
        }
        return $dates;
    }

    /**
     * Remove author dates from author string (used for using author names as search
     * term).
     *
     * @param string $author Author data
     *
     * @return mixed
     */
    public function removeAuthorDates($author)
    {
        $match = [];
        $regex =
            '/^(\s|.*)\s(fl.\s|d.\s|ca.\s|\*)*\s?(\d{4})\??(\sor\s\d\d?)?\s?'
            . '(-|–)?\s?(ca.\s|after\s|†)?(\d{1,4})?(.|,)?$/Uu';
        if (preg_match($regex, $author, $match)) {
            $author = (isset($match[1])) ? trim($match[1]) : $author;
        }
        // delete unnormalized characters of gallica ressource with source_id:20
        if (preg_match('/(.*)(\d.*)/Uus', $author, $match)) {
            $author = (isset($match[1])) ? trim($match[1]) : $author;
        }
        return $author;
    }

    /**
     * Rewrite links if defined
     *
     * @param array $links List with links schema [url] and [desc] for description.
     *
     * @return array $links Return processed links.
     */
    protected function rewriteLinks($links = [])
    {
        // if configuration empty return unprocessed
        if (!isset($this->config->LinksRewrite)
            || count($this->config->LinksRewrite) == 0
        ) {
            return $links;
        }

        // if links list empty return unprocessed
        if (count($links) == 0) {
            return $links;
        }

        foreach ($links as &$link) {
            $link = $this->rewriteLink($link);
        }

        return array_filter($links);
    }

    /**
     * Rewrite link
     *
     * @param string $link Link to rewrite
     *
     * @return string $link Return processed link.
     *
     * @to-do resolve non consistently behaviour between method pattern and remove
     *        to handle slashes in urls
     */
    protected function rewriteLink(&$link)
    {
        $rewrite = $this->config->LinksRewrite->toArray();
        foreach ($rewrite as $r) {
            // is remove pattern than suppress link refs #10834
            if (isset($r['remove'])) {
                if (0 != preg_match(
                    '/' . addcslashes($r['remove'], '/') . '/i',
                    trim($link['url'])
                )) {
                    unset($link);
                    return;
                }
            }
            // is pattern set and matches so try rewrite url
            if (isset($r['pattern']) && 0 != preg_match('/' . $r['pattern'] . '/i', trim($link['url']))) {
                // if function is set and available then perform on url
                if (isset($r['function']) && is_callable($r['function'])) {
                    $link['url'] = $r['function']($link['url']);
                }

                // is search and replace set so try to rewrite url
                if (isset($r['search']) && isset($r['replace'])) {
                    // prepare search pattern
                    // should be free of conflicting meta characters
                    $pattern = str_replace(['.'], ['\.'], $r['search']);
                    $pattern = '/(' . $pattern . ')/i';
                    // replace it only one time
                    $link['url'] = preg_replace(
                        $pattern,
                        trim($r['replace']),
                        trim($link['url']),
                        1,
                        $count
                    );
                    // add http if needed
                    // @to-do make it https compatible
                    if (!preg_match('/^(https?:\/\/)/', $link['url'])) {
                        $link['url'] = 'http://' . $link['url'];
                    }
                }
                // is method set so call alternatively method proceed link
                if (isset($r['method']) && method_exists($this, $r['method'])) {
                    $link['url'] = $this->$r['method']($link['url']);
                } // end if isset method
            } // end if isset pattern
        } // end foreach
        return $link;
    }

    /**
     * Resolve Rewrite/Schweitzer Url
     *
     * @param string $link Link to rewrite
     *
     * @return string $link Return processed link.
     */
    protected function resolveEblLink($link)
    {
        if (false === ($user = $this->manager->isLoggedIn())) {
            $id = $this->driver->getUniqueId();
            return $this->url->__invoke(
                'record-ebllink',
                [],
                ['query' => ['link' => $link, 'id' => $id]]
            );
        }
        $url = $this->rewrite->resolveLink($link, $user);
        return $url;
    }

    /**
     * Customized method for multi resolver support
     *
     * Get all the links associated with this record depending on the OpenURL setting
     * replace_other_urls.  Returns an array of associative arrays each containing
     * 'desc' and 'url' keys.
     *
     * @return bool
     */
    protected function hasOpenUrlReplaceSetting()
    {
        return isset($this->resolverConfig->General->replace_other_urls)
        && $this->resolverConfig->General->replace_other_urls;
    }

    /**
     * Get HTML to render a title.
     *
     * @param int $maxLength Maximum length of non-highlighted title.
     *
     * @return string
     */
    public function getTitleHtml($maxLength = 180)
    {
        $highlightedTitle = $this->driver->tryMethod('getHighlightedTitle');

        $titleRegexPattern = '/(\s[\/\.:]\s*)*$/';
        $shortTitle = preg_replace(
            $titleRegexPattern,
            '',
            trim($this->driver->tryMethod('getShortTitle'))
        );
        $subTitle = preg_replace(
            $titleRegexPattern,
            '',
            trim($this->driver->tryMethod('getSubTitle'))
        );
        $sectionTitle = preg_replace(
            $titleRegexPattern,
            '',
            trim($this->driver->tryMethod('getTitleSection'))
        );
        $title = (!empty($shortTitle) ? $shortTitle : preg_replace(
            $titleRegexPattern,
            '',
            trim($this->driver->tryMethod('getTitle'))
        )) .
        (!empty($subTitle) ? ': ' . $subTitle : '') .
        (!empty($sectionTitle) ? ', ' . $sectionTitle : '');

        if (!empty($highlightedTitle)) {
            $highlight = $this->getView()->plugin('highlight');
            $addEllipsis = $this->getView()->plugin('addEllipsis');
            return $highlight($addEllipsis($highlightedTitle, $title));
        }
        if (!empty($title)) {
            $escapeHtml = $this->getView()->plugin('escapeHtml');
            $truncate = $this->getView()->plugin('truncate');
            return $escapeHtml($truncate($title, $maxLength));
        }
        $transEsc = $this->getView()->plugin('transEsc');
        return $transEsc('Title not available');
    }

    /**
     * Get online editions
     *
     * @return array
     */
    public function getOnlineEditions()
    {
        $online_keys = [
            "Online-Ausg.",
            "Online-Ausg.:",
            "Online-Ausgabe",
            "Digital. Ausg.",
            "Online-Ausg. u.d.T.",
            "Elektronische Reproduktion"
        ];
        $onlineEditions = [];
        $recordLinkHelper = $this->getView()->plugin('recordLink');
        foreach ($this->driver->tryMethod('getAdditionals') as $add) {
            if (
                isset($add['identifier'])
                &&
                (
                    in_array($add['identifier'], $online_keys)
                    ||
                    (
                        isset($add['note'])
                        &&
                        in_array($add['note'], $online_keys)
                    )
                )
            ) {
                if (isset($add['id'])) {
                    $link = $recordLinkHelper->getTabUrl($add['id'], 'Holdings');
                    if (!empty($link)) {
                        $add['link'] = $link;
                        $onlineEditions[] = $add;
                    }
                } elseif (isset($add['related_records'])) {
                    foreach ($add['related_records']['first_results'] as $linkedRecord) {
                        $link = $recordLinkHelper->getTabUrl($linkedRecord);
                        if (!empty($link)) {
                            //overwrite link in entry and add to result array
                            $add['link'] = $link;
                            $onlineEditions[] = $add;
                        }
                    }
                }
            }
        }
        return $onlineEditions;
    }

    public function getAdvancedSearchQueryParams()
    {
        $queryParams = [];
        if ($title = $this->driver->getShortTitle()) {
            $queryParams['lookfor0'][] = $title;
            $queryParams['type0'][] = 'Title';
        }
        $isn = $this->driver->tryMethod('getCleanISSN');
        if (!empty($isn)) {
            $queryParams['lookfor0'][] = $isn;
            $queryParams['type0'][] = 'ISN';
        }
        if ($author = $this->driver->tryMethod('getPrimaryAuthors')) {
            $queryParams['lookfor0'][] = is_array($author) ? current($author) : $author;
            $queryParams['type0'][] = 'Author';
        }
        if (!empty($queryParams)) {
            // although this is a standard parameter
            // we should only pass it if there is an actual query to perform
            $queryParams['join'] = 'AND';
        }
        return $queryParams;
    }

    /**
     * Get the rendered cover plus some useful parameters.
     *
     * @deprecated used for VF7 ajaxcovers, remove on upgrade
     * @param string $context Context of code being generated
     * @param string $default The default size of the cover
     * @param string $link    The link for the anchor
     *
     * @return array
     */
    public function getCoverDetails($context, $default, $link = false)
    {
        $details = compact('link', 'context') + [
                'driver' => $this->driver, 'cover' => false, 'size' => false,
                'linkPreview' => $this->getPreviewCoverLinkSetting($context),
            ];
        $preferredSize = $this->getCoverSize($context, $default);
        if (empty($preferredSize)) {    // covers disabled entirely
            $details['html'] = '';
        } else {
            // Find best option if more than one size is defined (e.g. small:medium)
            foreach (explode(':', $preferredSize) as $size) {
                if ($details['cover'] = $this->getThumbnail($size)) {
                    $details['size'] = $size;
                    break;
                }
            }
            if ($details['size'] === false) {
                // we need to set a size for the ajax covers to work correctly
                list($details['size']) = explode(':', $preferredSize);
            }

            $details['html'] = $this->contextHelper->renderInContext(
                'record/cover.phtml', $details
            );
        }
        return $details;
    }

    /**
     * Generate a thumbnail URL (return false if unsupported).
     *
     * @deprecated used for VF7 ajaxcovers, remove on upgrade
     * @param string $size Size of thumbnail (small, medium or large -- small is
     * default).
     *
     * @return string|bool
     */
    public function getThumbnail($size = 'small')
    {
        // Find out whether or not AJAX covers are enabled; this will control
        // whether dynamic URLs are resolved immediately or deferred until later
        // (see third parameter of getUrl() below).
        $ajaxcovers = $this->config->Content->ajaxcovers ?? false;
        return $this->coverRouter
            ? $this->coverRouter->getUrl($this->driver, $size, !$ajaxcovers)
            : false;
    }
}
